/* Copyright (c) Michal Altair Valasek - Altairis, 2006 | www.altairis.cz
 * 
 * This program is free software licensed under terms of the BSD license:
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name of Michal Altair Valasek, Altairis nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

using System;
using System.Collections.Generic;
using System.Text;
using System.IO.Ports;

namespace Papouch.Thermometers {

    /// <summary>
    /// This class represents the "TM" serial thermometer from PaPouch Electronic.
    /// </summary>
    public class SerialSensor : ISensor {
        private const string SerialDataFormat = @"^[\+\-]\d{3}\.\d{1}C$";   // expected reply format for serial sensor
        private string portName;                                            // name of serial port
        private int readTimeout = 5000;                                     // read timeout in miliseconds

        /// <summary>
        /// Gets or sets serial communication read timeout in miliseconds. Default value is <c>5000</c> (5 seconds).
        /// </summary>
        public int ReadTimeout {
            get { return readTimeout; }
            set {
                if (value < 1000) throw new ArgumentException("ReadTimeout must be at least 1000 miliseconds.");
                readTimeout = value;
            }
        }

        /// <summary>
        /// Gets or sets name of serial port to read value from, ie. "<c>COM1</c>".
        /// </summary>
        public string PortName {
            get { return portName; }
            set {
                if (value == null) throw new ArgumentNullException();
                if (!System.Text.RegularExpressions.Regex.IsMatch(value, "^COM[0-9]{1,}$")) throw new ArgumentException("Port name must be in format 'COMx' where x is number.");
                portName = value;
            }
        }

        /// <summary>
        /// Initializes instance of this class.
        /// </summary>
        /// <param name="portName">Name of serial port to read value from, ie. "<c>COM1</c>".</param>
        public SerialSensor(string portName) {
            this.PortName = portName;
        }

        // ISensor implementation
        float ISensor.ReadSample() {
            // Open port
            SerialPort port = new SerialPort(this.PortName, 9600, Parity.None, 8, StopBits.One);
            port.ReadTimeout = this.ReadTimeout;
            port.NewLine = "\x0d"; // CR
            string td = null;

            // Read data
            try {
                port.Open();
                port.DtrEnable = true;
                td = port.ReadLine();
            }
            finally {
                port.Close();
            }

            // Parse data
            if (!System.Text.RegularExpressions.Regex.IsMatch(td, SerialDataFormat)) throw new FormatException(string.Format("Sensor returned data in unexpected format: '{0}'.", td));
            return float.Parse(td.Substring(0, 6), System.Globalization.NumberStyles.Float, System.Globalization.CultureInfo.InvariantCulture);
        }
    }
}
